/*------------------------------------------------------------------------------
-- This file is a part of the QLop Software
-- Copyright (C) 2015, Plasma Physics Laboratory - CNRS
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
-------------------------------------------------------------------------------*/
/*-- Author : Alexis Jeandet
-- Mail : alexis.jeandet@member.fsf.org
----------------------------------------------------------------------------*/
#include "qlopnetworkproxyfactory.h"
#include <QHostInfo>
#include <qlopsettings.h>
#include <filedownloader.h>

QLopNetworkProxyFactory::QLopNetworkProxyFactory()
    :QNetworkProxyFactory()
{
    p_pacParser = new ProxyPacParser();
}

QLopNetworkProxyFactory::~QLopNetworkProxyFactory()
{

}


QList<QNetworkProxy> QLopNetworkProxyFactory::queryProxy(const QNetworkProxyQuery &query)
{
    QList<QNetworkProxy> result;
    switch (proxyCfgType)
    {
    case none:
        result << QNetworkProxy(QNetworkProxy::NoProxy);
        return result;
        break;
    case manual:
        return queryProxy_manual(query);
        break;
    case system:
        return queryProxy_system(query);
        break;
    case automatic:
        return queryProxy_auto(query);
        break;
    default:
        result << QNetworkProxy(QNetworkProxy::DefaultProxy);
        return result;
        break;
    }
}


//should hanlde ignored hosts
QList<QNetworkProxy> QLopNetworkProxyFactory::queryProxy_manual(const QNetworkProxyQuery &query)
{
    QList<QNetworkProxy> result;
    QString scheme = query.url().scheme();
    QHostInfo peerHost = QHostInfo::fromName(query.peerHostName());
    if(scheme=="http")
    {
        result.append(p_http_proxy);
    }
    if(scheme=="https")
    {
        result.append(p_https_proxy);
    }
    if(scheme=="ftp")
    {
        result.append(p_ftp_proxy);
    }
    result << QNetworkProxy(QNetworkProxy::DefaultProxy);
    return result;
}

QList<QNetworkProxy> QLopNetworkProxyFactory::queryProxy_system(const QNetworkProxyQuery &query)
{
    QList<QNetworkProxy> result;
    result << QNetworkProxy(QNetworkProxy::DefaultProxy);
    return result;
}

QList<QNetworkProxy> QLopNetworkProxyFactory::queryProxy_auto(const QNetworkProxyQuery &query)
{
    QList<QNetworkProxy> result;
    QString config=p_pacParser->findProxyForUrl(query.url().toString(),QHostInfo::localHostName());
    QStringList proxyList=config.split(";");
    for(int i=0;i<proxyList.count();i++)
    {
        QStringList line = proxyList.at(i).split(" ");
        if(line.count()>=2)
        { //TODO check correct syntax for pac returned proxy cfg
            if(line.at(0).compare("PROXY",Qt::CaseInsensitive))
            {
                QStringList proxyArgs=line.at(1).split(":");
                if(proxyArgs.count()>=2)
                {
                    result << QNetworkProxy(QNetworkProxy::HttpProxy,proxyArgs.at(0),proxyArgs.at(1).toUInt());
                }
            }
            else if (line.at(0).compare("SOCKS",Qt::CaseInsensitive))
            {
                QStringList proxyArgs=line.at(1).split(":");
                if(proxyArgs.count()>=2)
                {
                    result << QNetworkProxy(QNetworkProxy::Socks5Proxy,proxyArgs.at(0),proxyArgs.at(1).toUInt());
                }
            }
            else
            {
                result << QNetworkProxy(QNetworkProxy::NoProxy);
            }
        }
        else
        {
            result << QNetworkProxy(QNetworkProxy::NoProxy);
        }
    }
    return result;
}

void QLopNetworkProxyFactory::reloadConfig()
{
    QString proxyType = QLopSettings::value(FileDownloader::self(),CFG_PROXY_TYPE_ENTRY,CFG_PROXY_TYPE_VALUE_NONE).toString();
    if(!proxyType.compare(CFG_PROXY_TYPE_VALUE_NONE))
    {
        this->proxyCfgType = none;
    }
    else
        if(!proxyType.compare(CFG_PROXY_TYPE_VALUE_MANUAL))
        {
            this->proxyCfgType = manual;
            QString host = QLopSettings::value(FileDownloader::self(),CFG_PROXY_HTTP_HOST_ENTRY).toString();
            QString port = QLopSettings::value(FileDownloader::self(),CFG_PROXY_HTTP_PORT_ENTRY).toString();
            this->p_http_proxy = QNetworkProxy(QNetworkProxy::HttpProxy,host,port.toUInt());

            host = QLopSettings::value(FileDownloader::self(),CFG_PROXY_HTTPS_HOST_ENTRY).toString();
            port = QLopSettings::value(FileDownloader::self(),CFG_PROXY_HTTPS_PORT_ENTRY).toString();
            this->p_https_proxy = QNetworkProxy(QNetworkProxy::HttpProxy,host,port.toUInt());

            host = QLopSettings::value(FileDownloader::self(),CFG_PROXY_FTP_HOST_ENTRY).toString();
            port = QLopSettings::value(FileDownloader::self(),CFG_PROXY_FTP_PORT_ENTRY).toString();
            this->p_ftp_proxy = QNetworkProxy(QNetworkProxy::HttpProxy,host,port.toUInt());

            host = QLopSettings::value(FileDownloader::self(),CFG_PROXY_SOCKS_HOST_ENTRY).toString();
            port = QLopSettings::value(FileDownloader::self(),CFG_PROXY_SOCKS_PORT_ENTRY).toString();
            this->p_socks_proxy = QNetworkProxy(QNetworkProxy::Socks5Proxy,host,port.toUInt());

        }
        else
            if(!proxyType.compare(CFG_PROXY_TYPE_VALUE_SYSTEM))
            {
                this->proxyCfgType = system;
            }
            else
                if(!proxyType.compare(CFG_PROXY_TYPE_VALUE_AUTOMATIC))
                {
                    this->proxyCfgType = automatic;
                }
}
